package frostfsid

import (
	"testing"

	"git.frostfs.info/TrueCloudLab/frostfs-node/internal/ape"
	"github.com/stretchr/testify/require"
)

func TestNamespaceRegexp(t *testing.T) {
	for _, tc := range []struct {
		name      string
		namespace string
		matched   bool
	}{
		{
			name:      "root empty ns",
			namespace: "",
			matched:   true,
		},
		{
			name:      "simple valid ns",
			namespace: "my-namespace-123",
			matched:   true,
		},
		{
			name:      "root placeholder",
			namespace: "<root>",
			matched:   false,
		},
		{
			name:      "too long",
			namespace: "abcdefghijklmnopkrstuvwxyzabcdefghijklmnopkrstuvwxyz",
			matched:   false,
		},
		{
			name:      "start with hyphen",
			namespace: "-ns",
			matched:   false,
		},
		{
			name:      "end with hyphen",
			namespace: "ns-",
			matched:   false,
		},
		{
			name:      "with spaces",
			namespace: "ns ns",
			matched:   false,
		},
	} {
		t.Run(tc.name, func(t *testing.T) {
			require.Equal(t, tc.matched, ape.NamespaceNameRegexp.MatchString(tc.namespace))
		})
	}
}

func TestSubjectNameRegexp(t *testing.T) {
	for _, tc := range []struct {
		name    string
		subject string
		matched bool
	}{
		{
			name:    "empty",
			subject: "",
			matched: false,
		},
		{
			name:    "invalid",
			subject: "invalid{name}",
			matched: false,
		},
		{
			name:    "too long",
			subject: "abcdefghijklmnopkrstuvwxyzabcdefghijklmnopkrstuvwxyzabcdefghijklmnopkrstuvwxyz",
			matched: false,
		},
		{
			name:    "valid",
			subject: "valid_name.012345@6789",
			matched: true,
		},
	} {
		t.Run(tc.name, func(t *testing.T) {
			require.Equal(t, tc.matched, ape.SubjectNameRegexp.MatchString(tc.subject))
		})
	}
}

func TestSubjectGroupRegexp(t *testing.T) {
	for _, tc := range []struct {
		name    string
		subject string
		matched bool
	}{
		{
			name:    "empty",
			subject: "",
			matched: false,
		},
		{
			name:    "invalid",
			subject: "invalid{name}",
			matched: false,
		},
		{
			name:    "too long",
			subject: "abcdefghijklmnopkrstuvwxyzabcdefghijklmnopkrstuvwxyzabcdefghijklmnopkrstuvwxyzabcdefghijklmnopkrstuvwxyzabcdefghijklmnopkrstuvwxyz",
			matched: false,
		},
		{
			name:    "long",
			subject: "abcdefghijklmnopkrstuvwxyzabcdefghijklmnopkrstuvwxyzabcdefghijklmnopkrstuvwxyz",
			matched: true,
		},
		{
			name:    "valid",
			subject: "valid_name.012345@6789",
			matched: true,
		},
	} {
		t.Run(tc.name, func(t *testing.T) {
			require.Equal(t, tc.matched, ape.GroupNameRegexp.MatchString(tc.subject))
		})
	}
}
